/*
 *  Copyright  2016 Harry Sintonen / MorphOS Team
 */
#define AROS_ALMOST_COMPATIBLE 1
#include <string.h>
#include <exec/execbase.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include "liblist_version.h"

static int scanlibs(struct ExecBase *SysBase, struct MinList *list);
static void freelibs(struct ExecBase *SysBase, struct MinList *list);

struct libnode
{
	struct MinNode node;
	APTR address;
	UWORD version;
	UWORD revision;
	UWORD opencnt;
	UBYTE flags;
	UBYTE name[1];
};

static const UBYTE version[] __attribute__((used)) = VERSTAG;

int main(void)
{
	struct ExecBase *SysBase = *(struct ExecBase **) 4;
	struct DosLibrary *DOSBase = (APTR) OpenLibrary("dos.library", 50);
	int rc = RETURN_OK;
	if (DOSBase)
	{
		STRPTR name = NULL;
		APTR rda = ReadArgs("NAME", (LONG *) &name, NULL);
		if (rda)
		{
			STRPTR pat = name ? name : (STRPTR) "#?";
			UBYTE parsed[strlen(pat) * 2 + 2];

			if (ParsePatternNoCase(pat, parsed, strlen(pat) * 2 + 2) >= 0)
			{
				struct MinList libs;

				NEWLIST(&libs);
				if (scanlibs(SysBase, &libs))
				{
					struct libnode *node;

					Printf("Address     Version     Rev.  OpenCnt  Flags        Name\n"
					       "----------  -------  -------  -------  -----------  -------------------------------\n");

					ForeachNode(&libs, node)
					{
						if (!MatchPatternNoCase((STRPTR) parsed, (STRPTR) node->name))
							continue;

						Printf("0x%08.8lx  %7.7lu  %7.7lu  %7.7lu  %lc%lc%lc%lc%lc%lc 0x%02.2lx  %s\n",
							(ULONG)node->address,
							(ULONG)node->version,
							(ULONG)node->revision,
							(ULONG)node->opencnt,
							(ULONG)((node->flags & LIBF_SUMMING)   ? 'S' : '-'),
							(ULONG)((node->flags & LIBF_CHANGED)   ? 'C' : '-'),
							(ULONG)((node->flags & LIBF_SUMUSED)   ? 'U' : '-'),
							(ULONG)((node->flags & LIBF_DELEXP )   ? 'D' : '-'),
							(ULONG)((node->flags & LIBF_RAMLIB)    ? 'R' : '-'),
							(ULONG)((node->flags & LIBF_QUERYINFO) ? 'Q' : '-'),
							(ULONG)node->flags,
							(ULONG)node->name);

						if (CheckSignal(SIGBREAKF_CTRL_C))
						{
							rc = RETURN_ERROR;
							SetIoErr(ERROR_BREAK);
							break;
						}
					}
				}
				else
					rc = RETURN_FAIL;

				freelibs(SysBase, &libs);
			}
			else
				rc = RETURN_FAIL;

			FreeArgs(rda);
		}
		else
			rc = RETURN_ERROR;

		if (rc)
			PrintFault(IoErr(), "LibList");

		CloseLibrary((APTR) DOSBase);
	}

	return rc;
}

static int scanlibs(struct ExecBase *SysBase, struct MinList *list)
{
	struct Library *l;
	int rc = 1;

	Forbid();

	ForeachNode(&SysBase->LibList, l)
	{
		struct libnode *node = AllocVec(sizeof(*node) + strlen(l->lib_Node.ln_Name), MEMF_ANY);
		if (!node)
		{
			rc = 0;
			break;
		}
		node->address  = l;
		node->version  = l->lib_Version;
		node->revision = l->lib_Revision;
		node->opencnt  = l->lib_OpenCnt;
		node->flags    = l->lib_Flags;
		strcpy((char *) node->name, (const char *) l->lib_Node.ln_Name);
		ADDTAIL(list, &node->node);
	}

	Permit();

	return rc;
}

static void freelibs(struct ExecBase *SysBase, struct MinList *list)
{
	struct libnode *n, *n2;

	ForeachNodeSafe(list, n, n2)
	{
		FreeVec(n);
	}
}
